package kr.co.vividnext.sodalive.app

import android.app.Application
import android.content.pm.ApplicationInfo
import android.content.pm.PackageManager
import android.os.Build
import androidx.appcompat.app.AppCompatDelegate
import com.appsflyer.AppsFlyerLib
import com.appsflyer.deeplink.DeepLinkResult
import com.facebook.FacebookSdk
import com.orhanobut.logger.AndroidLogAdapter
import com.orhanobut.logger.Logger
import kr.co.vividnext.sodalive.BuildConfig
import kr.co.vividnext.sodalive.common.ImageLoaderProvider
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.di.AppDI
import kr.co.vividnext.sodalive.tracking.FirebaseTracking
import tech.notifly.Notifly

class SodaLiveApp : Application() {
    override fun onCreate() {
        super.onCreate()

        Logger.addLogAdapter(object : AndroidLogAdapter() {
            override fun isLoggable(priority: Int, tag: String?): Boolean {
                return BuildConfig.DEBUG && isDebuggable()
            }
        })

        AppDI(applicationContext, BuildConfig.DEBUG && isDebuggable())

        AppCompatDelegate.setDefaultNightMode(AppCompatDelegate.MODE_NIGHT_YES)

        SharedPreferenceManager.init(applicationContext)

        ImageLoaderProvider.init(applicationContext)

        FacebookSdk.fullyInitialize()

        setupAppsFlyer()

        setupNotifly()
    }

    private fun isDebuggable(): Boolean {
        var debuggable = false

        try {
            val appInfo = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                packageManager.getApplicationInfo(
                    packageName,
                    PackageManager.ApplicationInfoFlags.of(0L)
                )
            } else {
                packageManager.getApplicationInfo(packageName, 0)
            }
            debuggable = 0 != appInfo.flags and ApplicationInfo.FLAG_DEBUGGABLE
        } catch (e: PackageManager.NameNotFoundException) {
            /* debuggable variable will remain false */
        }

        return debuggable
    }

    private fun setupAppsFlyer() {
        // Appsflyer SDK 초기화
        AppsFlyerLib.getInstance().init("tWF2wbJ5nSkya5Ru9mGcPU", null, this)
        AppsFlyerLib.getInstance().start(this)

        // 딥링크 및 디퍼드 딥링크 처리
        AppsFlyerLib.getInstance().subscribeForDeepLink { deepLinkResult ->
            when (deepLinkResult.status) {
                DeepLinkResult.Status.FOUND -> {
                    SharedPreferenceManager.alreadyTrackingAppLaunch = false

                    val deepLink = deepLinkResult.deepLink
                    SharedPreferenceManager.marketingLinkValue = deepLink?.getStringValue(
                        "deep_link_value"
                    ) ?: ""

                    val marketingPid = deepLink?.getStringValue(
                        "deep_link_sub1"
                    )

                    if (marketingPid != null) {
                        SharedPreferenceManager.marketingPid = marketingPid
                    }

                    SharedPreferenceManager.marketingUtmSource = deepLink?.getStringValue(
                        "deep_link_sub2"
                    ) ?: ""
                    SharedPreferenceManager.marketingUtmMedium = deepLink?.getStringValue(
                        "deep_link_sub3"
                    ) ?: ""
                    SharedPreferenceManager.marketingUtmCampaign = deepLink?.getStringValue(
                        "deep_link_sub4"
                    ) ?: ""
                    SharedPreferenceManager.marketingLinkValueId = deepLink?.getStringValue(
                        "deep_link_sub5"
                    )?.toLongOrNull() ?: 0L

                    logUtmInFirebase()
                }

                DeepLinkResult.Status.NOT_FOUND -> Logger.d("딥링크를 찾을 수 없습니다.")
                DeepLinkResult.Status.ERROR -> Logger.d("딥링크 처리 중 오류 발생: ${deepLinkResult.error}")
            }
        }
    }

    private fun logUtmInFirebase() {
        FirebaseTracking.logUtm()
    }

    private fun setupNotifly() {
        Notifly.initialize(
            applicationContext,
            BuildConfig.NOTIFLY_PROJECT_ID,
            BuildConfig.NOTIFLY_USERNAME,
            BuildConfig.NOTIFLY_PASSWORD,
        )
    }
}
