package kr.co.vividnext.sodalive.user.login

import android.app.Service
import android.content.Intent
import android.graphics.Rect
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.transition.TransitionManager
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.LinearLayout
import android.widget.Toast
import androidx.annotation.OptIn
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.ConstraintSet
import androidx.media3.common.util.UnstableApi
import com.jakewharton.rxbinding4.widget.textChanges
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.databinding.ActivityLoginBinding
import kr.co.vividnext.sodalive.extensions.dpToPx
import kr.co.vividnext.sodalive.main.MainActivity
import kr.co.vividnext.sodalive.user.find_password.FindPasswordActivity
import kr.co.vividnext.sodalive.user.signup.SignUpActivity
import org.koin.android.ext.android.inject

@OptIn(UnstableApi::class)
class LoginActivity : BaseActivity<ActivityLoginBinding>(ActivityLoginBinding::inflate) {

    private val viewModel: LoginViewModel by inject()

    private lateinit var imm: InputMethodManager
    private lateinit var loadingDialog: LoadingDialog

    private val handler = Handler(Looper.getMainLooper())

    override fun onCreate(savedInstanceState: Bundle?) {
        imm = getSystemService(Service.INPUT_METHOD_SERVICE) as InputMethodManager

        super.onCreate(savedInstanceState)
        bindData()
    }

    override fun setupView() {
        binding.root.viewTreeObserver.addOnGlobalLayoutListener {
            val rect = Rect()
            binding.root.getWindowVisibleDisplayFrame(rect)

            val keypadHeight = screenHeight - rect.bottom
            if (keypadHeight > screenHeight * 0.15) {
                updateMarginWithAnimation(
                    binding.llLoginContainer,
                    -100f.dpToPx().toInt()
                )
            } else {
                updateMarginWithAnimation(binding.llLoginContainer, 0)
            }
        }

        binding.toolbar.tvBack.text = "로그인"
        binding.toolbar.tvBack.setOnClickListener { finish() }
        loadingDialog = LoadingDialog(this, layoutInflater)

        binding.etPassword.setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_DONE) {
                hideKeyboard()
                login()
                true
            } else {
                false
            }
        }
        binding.tvLogin.setOnClickListener {
            hideKeyboard()
            login()
        }

        binding.tvSignUp.setOnClickListener {
            val nextIntent = Intent(applicationContext, SignUpActivity::class.java)
            val extras = intent.getBundleExtra(Constants.EXTRA_DATA)
                ?: if (intent.extras != null) {
                    intent.extras
                } else {
                    null
                }
            if (extras != null) {
                nextIntent.putExtra(Constants.EXTRA_DATA, extras)
            }
            startActivity(nextIntent)
        }

        binding.tvForgotPassword.setOnClickListener {
            startActivity(
                Intent(
                    applicationContext,
                    FindPasswordActivity::class.java
                )
            )
        }
    }

    private fun login() {
        viewModel.login {
            it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
            finishAffinity()
            val nextIntent = Intent(applicationContext, MainActivity::class.java)
            val extras = intent.getBundleExtra(Constants.EXTRA_DATA)
                ?: if (intent.extras != null) {
                    intent.extras
                } else {
                    null
                }
            if (extras != null) {
                nextIntent.putExtra(Constants.EXTRA_DATA, extras)
            }
            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
            intent.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP)
            startActivity(nextIntent)
        }
    }

    private fun bindData() {
        compositeDisposable.add(
            binding.etEmail.textChanges().skip(1)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    viewModel.email = it.toString()
                }
        )

        compositeDisposable.add(
            binding.etPassword.textChanges().skip(1)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    viewModel.password = it.toString()
                }
        )

        viewModel.toastLiveData.observe(this) {
            it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(this) {
            if (it) {
                loadingDialog.show(screenWidth)
            } else {
                loadingDialog.dismiss()
            }
        }
    }

    private fun updateMarginWithAnimation(view: LinearLayout, newMargin: Int) {
        val constraintLayout = view.parent as ConstraintLayout
        val constraintSet = ConstraintSet()
        constraintSet.clone(constraintLayout)

        // 변경할 Constraint 적용 (margin 변경)
        constraintSet.connect(
            view.id,
            ConstraintSet.TOP,
            ConstraintSet.PARENT_ID,
            ConstraintSet.TOP,
            newMargin
        )

        // 애니메이션 적용
        TransitionManager.beginDelayedTransition(constraintLayout)
        constraintSet.applyTo(constraintLayout)
    }

    private fun hideKeyboard() {
        handler.postDelayed({
            imm.hideSoftInputFromWindow(
                window.decorView.applicationWindowToken,
                InputMethodManager.HIDE_NOT_ALWAYS
            )
        }, 100)
    }
}
