package kr.co.vividnext.sodalive.user.signup

import android.app.Service
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import androidx.annotation.OptIn
import androidx.media3.common.util.UnstableApi
import com.jakewharton.rxbinding4.widget.textChanges
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.databinding.ActivitySignupBinding
import kr.co.vividnext.sodalive.main.MainActivity
import kr.co.vividnext.sodalive.settings.terms.TermsActivity
import org.koin.android.ext.android.inject

@OptIn(UnstableApi::class)
class SignUpActivity : BaseActivity<ActivitySignupBinding>(ActivitySignupBinding::inflate) {

    private val viewModel: SignUpViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var imm: InputMethodManager

    private val handler = Handler(Looper.getMainLooper())

    override fun onCreate(savedInstanceState: Bundle?) {
        imm = getSystemService(Service.INPUT_METHOD_SERVICE) as InputMethodManager

        super.onCreate(savedInstanceState)
        bindData()
    }

    override fun setupView() {
        binding.toolbar.tvBack.text = "회원가입"
        binding.toolbar.tvBack.setOnClickListener { finish() }
        loadingDialog = LoadingDialog(this, layoutInflater)

        binding.etEmail.post {
            binding.etEmail.requestFocus()
            imm.showSoftInput(binding.etEmail, InputMethodManager.SHOW_IMPLICIT)
        }

        binding.tvTermsOfService.setOnClickListener {
            val intent = Intent(applicationContext, TermsActivity::class.java)
            intent.putExtra(Constants.EXTRA_TERMS, Constants.EXTRA_TERMS)
            startActivity(intent)
        }

        binding.tvPrivacyPolicy.setOnClickListener {
            val intent = Intent(applicationContext, TermsActivity::class.java)
            intent.putExtra(Constants.EXTRA_TERMS, Constants.EXTRA_PRIVACY)
            startActivity(intent)
        }

        binding.rlTermsOfService.setOnClickListener { viewModel.onClickCheckboxTermsOfService() }
        binding.ivTermsOfService.setOnClickListener { viewModel.onClickCheckboxTermsOfService() }

        binding.rlPrivacyPolicy.setOnClickListener { viewModel.onClickCheckboxPrivacyPolicy() }
        binding.ivPrivacyPolicy.setOnClickListener { viewModel.onClickCheckboxPrivacyPolicy() }

        binding.tvSignUp.setOnClickListener {
            hideKeyboard()
            viewModel.signUp {
                it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
                finishAffinity()
                val nextIntent = Intent(applicationContext, MainActivity::class.java)
                val extras = intent.getBundleExtra(Constants.EXTRA_DATA)
                    ?: if (intent.extras != null) {
                        intent.extras
                    } else {
                        null
                    }
                if (extras != null) {
                    nextIntent.putExtra(Constants.EXTRA_DATA, extras)
                }
                startActivity(nextIntent)
            }
        }
    }

    private fun bindData() {
        compositeDisposable.add(
            binding.etEmail.textChanges().skip(1)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    viewModel.email = it.toString()
                }
        )

        compositeDisposable.add(
            binding.etPassword.textChanges().skip(1)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    viewModel.password = it.toString()
                }
        )

        viewModel.isAgreeTermsOfServiceLiveData.observe(this) {
            binding.ivTermsOfService.isSelected = it
        }

        viewModel.isAgreePrivacyPolicyLiveData.observe(this) {
            binding.ivPrivacyPolicy.isSelected = it
        }

        viewModel.signUpErrorLiveData.observe(this) {
            Toast.makeText(applicationContext, it.message, Toast.LENGTH_LONG).show()

            when (it.errorProperty) {
                "email" -> {
                    binding.etEmail.error = it.message
                    binding.etEmail.requestFocus()
                }

                "password" -> {
                    binding.etPassword.error = it.message
                    binding.etPassword.requestFocus()
                }
            }
        }

        viewModel.isLoading.observe(this) {
            if (it) {
                loadingDialog.show(screenWidth)
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.toastLiveData.observe(this) {
            it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
        }
    }

    private fun hideKeyboard() {
        handler.postDelayed({
            imm.hideSoftInputFromWindow(
                window.decorView.applicationWindowToken,
                InputMethodManager.HIDE_NOT_ALWAYS
            )
        }, 100)
    }
}
