package kr.co.vividnext.sodalive.user.signup

import androidx.core.util.PatternsCompat
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.tracking.FirebaseTracking
import kr.co.vividnext.sodalive.user.UserRepository

class SignUpViewModel(private val repository: UserRepository) : BaseViewModel() {
    var email = ""
    var password = ""

    private val _isAgreeTermsOfServiceLiveData = MutableLiveData(false)
    val isAgreeTermsOfServiceLiveData: LiveData<Boolean>
        get() = _isAgreeTermsOfServiceLiveData

    private val _isAgreePrivacyPolicyLiveData = MutableLiveData(false)
    val isAgreePrivacyPolicyLiveData: LiveData<Boolean>
        get() = _isAgreePrivacyPolicyLiveData

    private val _signUpErrorLiveData = MutableLiveData<SignUpError>()
    val signUpErrorLiveData: LiveData<SignUpError>
        get() = _signUpErrorLiveData

    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    fun signUp(onSuccess: (String?) -> Unit) {
        if (!validation()) return
        val request = SignUpRequest(
            email = email,
            password = password,
            marketingPid = SharedPreferenceManager.marketingPid,
            isAgreeTermsOfService = _isAgreeTermsOfServiceLiveData.value!!,
            isAgreePrivacyPolicy = _isAgreePrivacyPolicyLiveData.value!!
        )

        _isLoading.value = true
        compositeDisposable.add(
            repository.signUp(request)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        _isLoading.value = false
                        if (it.success && it.data != null) {
                            SharedPreferenceManager.token = it.data.token
                            SharedPreferenceManager.email = it.data.email
                            SharedPreferenceManager.userId = it.data.userId
                            SharedPreferenceManager.nickname = it.data.nickname
                            SharedPreferenceManager.profileImage = it.data.profileImage
                            onSuccess(it.message)
                            FirebaseTracking.signUp("email")
                        } else {
                            if (it.errorProperty != null && it.message != null) {
                                _signUpErrorLiveData.postValue(
                                    SignUpError(it.errorProperty, it.message)
                                )
                            } else if (it.message != null) {
                                _toastLiveData.postValue(it.message)
                            } else {
                                _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                            }
                        }
                    },
                    {
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }

    private fun validation(): Boolean {
        if (email.isBlank()) {
            _signUpErrorLiveData.postValue(
                SignUpError(
                    "email",
                    "이메일을 입력하세요."
                )
            )

            return false
        }

        if (password.isBlank()) {
            _signUpErrorLiveData.postValue(
                SignUpError(
                    "password",
                    "비밀번호를 입력하세요."
                )
            )

            return false
        }

        if (
            !_isAgreePrivacyPolicyLiveData.value!! ||
            !_isAgreeTermsOfServiceLiveData.value!!
        ) {
            _signUpErrorLiveData.postValue(
                SignUpError(
                    "",
                    "약관에 동의하셔야 회원가입이 가능합니다."
                )
            )

            return false
        }

        if (!PatternsCompat.EMAIL_ADDRESS.matcher(email).matches()) {
            _signUpErrorLiveData.postValue(
                SignUpError(
                    "email",
                    "올바른 이메일을 입력해 주세요"
                )
            )

            return false
        }

        if (
            "^(?=.*[A-Za-z])(?=.*\\d)[A-Za-z\\d$@!%*#?&]{8,}$"
                .toRegex()
                .matches(password)
                .not()
        ) {
            _signUpErrorLiveData.postValue(
                SignUpError(
                    "password",
                    "영문, 숫자 포함 8자 이상의 비밀번호를 입력해 주세요."
                )
            )

            return false
        }

        return true
    }

    fun onClickCheckboxTermsOfService() {
        _isAgreeTermsOfServiceLiveData.postValue(!_isAgreeTermsOfServiceLiveData.value!!)
    }

    fun onClickCheckboxPrivacyPolicy() {
        _isAgreePrivacyPolicyLiveData.postValue(!_isAgreePrivacyPolicyLiveData.value!!)
    }
}
